import * as style from "./CoursesTab.module.css"
import React, { useContext, forwardRef } from "react";
import { Context } from "../../state/state";
import MultiSelect from "../MultiSelect";
import DatePicker from "react-datepicker";
import ru from "date-fns/locale/ru";
import Spinner from "../Spinner"
import { tr } from "date-fns/locale";
import SegmentBtn from "../SegmentBtn";
import SegmentSwitch from "../SegmentSwitch";
import SelectSegment from "../SelectSegment/SelectSegment";
import Warning from "../Warning";
import ErrorNotice from "../ErrorNotice";
import UserLink from "../UserLink";

export default function CoursesTab() {
    const { actions, state } = useContext(Context)
    const { onWarning, onError } = actions.courses;
    const { ssr, } = state
    const { course, dateFrom, dateTo, levels, items, isFetching, warning, error } = state.courses
    const { segment_tab, segment, is_segment } = state.courses
    const { courses, segment: { onClickSegmentDownload } } = actions

    async function submit() {
        let body;
        let is_segment = segment_tab === '2';
        if (!course) {
            onWarning('Укажите курс!')
            return;
        }
        if (segment_tab === '1') {
            if (levels.length === 0) {
                onWarning('Укажите уровни доступа!')
                return;
            }
            body = { dateTo, dateFrom, levels, course }
        } else {
            if (!segment) {
                onWarning('Укажите сегмент!')
                return;
            }
            body = { dateTo, dateFrom, course, segment }
        }

        actions.courses.onFetching()
        try {
            const res = await actions.fetch({
                route: 'courses',
                method: 'POST',
                body
            })
            if (res.ok) {
                actions.courses.onFetched({ ...res, is_segment })
            }
        } catch (e) {
            onError('Произошла ошибка. Повторите позже')
        } finally {
            actions.courses.onEnd()
        }
    }

    const coursesLevels = course !== '' ? ssr.courses.find(c => +c.term_id === +course).levels : []

    function columnSegment(item, column) {
        let segment = item.segment;

        return { ...segment, params: { ...segment.params, column } }
    }

    function showSegment(value) {
        return !is_segment && value !== 0 && value.toString().trim() !== '-'
    }

    function downloadUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.base_uri + '&downloadUsers=' + encodeURIComponent(params))
    }

    function viewUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.edit_uri + '&viewUsers=' + encodeURIComponent(params))
    }

    return <div>
        {warning !== '' &&
            <div className={style.warning}>
                <Warning message={warning}/>
            </div>
        }
        {error !== '' &&
            <div className={style.warning}>
                <ErrorNotice message={error}/>
            </div>
        }
        <div className={style.course}>
            <label htmlFor="course"><span
                className="dashicons dashicons-welcome-learn-more"></span> Курс</label>&nbsp;
            <select className={style.courseSelect} id="course" value={course}
                    onChange={courses.courseChanged}>
                <option value="">Выберите курс</option>
                {ssr.courses.map(({ name, term_id }, index) =>
                    <option key={index} value={term_id}>{name}</option>
                )}
            </select>
        </div>
        <SegmentSwitch active={segment_tab} onChange={courses.switchedSegmentTab}/>
        {segment_tab === '1'
            ? <div className={style.row}>
                <div className={style.left}>
                    <div className={style.level}>
                        <div>
                            <span className="dashicons dashicons-lock"></span> Уровень доступа
                        </div>
                        <MultiSelect
                            disabled={course === ''}
                            value={levels}
                            onChange={courses.levelsChanged}
                            name="level"
                            options={ssr.levels.filter(level => coursesLevels.includes(level.term_id))
                                .map(level => ({ val: level.term_id.toString(), label: level.name }))}
                        />
                    </div>
                </div>
            </div>
            : <SelectSegment value={segment} onChange={courses.segmentChange}/>
        }
        <div className={style.dates}>
                       <span>
                            <span className="dashicons dashicons-calendar-alt"></span> Доступ активен в период c&nbsp;
                       </span>
            <DatePicker
                placeholderText="__ /__ /____"
                locale={ru}
                dateFormat="dd.MM.yyyy"
                customInput={<DateInput/>}
                selected={dateFrom}
                onChange={courses.dateFromChanged}
            />
            &nbsp;по&nbsp;
            <DatePicker
                placeholderText="__ /__ /____"
                locale={ru}
                dateFormat="dd.MM.yyyy"
                customInput={<DateInput/>}
                selected={dateTo}
                onChange={courses.dateToChanged}
            />
        </div>
        <div className={style.brnWrap}>
            <button type="button" className="button button-primary" onClick={submit}>Применить</button>
        </div>
        <table border={1} className={style.table}>
            <thead>
            <tr>
                <th>Уроки</th>
                <th>Доступен</th>
                <th>Открывали</th>
                <th>Не открывали</th>
                <th>Отправили задание</th>
                <th>Не отправили задание</th>
                <th>Задание одобрено</th>
                <th>Задание проверяется</th>
                <th>Задание отклонено</th>
            </tr>
            </thead>
            <tbody>
            {isFetching
                ? <tr>
                    <td style={{ textAlign: 'center' }} colSpan={9}><Spinner/></td>
                </tr>
                : <>
                    {items.map(item =>
                        <tr key={item.id}>
                            <td>{item.name}</td>
                            <td className={style.num}>
                                {showSegment(item.available) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'available'))}>
                                            {item.available}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'available')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.available
                                }
                            </td>
                            <td className={style.num}>
                                {showSegment(item.opened) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'opened'))}>
                                            {item.opened}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'opened')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.opened
                                }
                            </td>
                            <td className={style.num}>
                                {showSegment(item.no_opened) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'no_opened'))}>
                                            {item.no_opened}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'no_opened')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.no_opened
                                }
                            </td>
                            <td className={style.num}>

                                {showSegment(item.homework.sent) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'homework_sent'))}>
                                            {item.homework.sent}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'homework_sent')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.homework.sent
                                }
                            </td>
                            <td className={style.num}>
                                {showSegment(item.homework.no_sent) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'homework_no_sent'))}>
                                            {item.homework.no_sent}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'homework_no_sent')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.homework.no_sent
                                }
                            </td>
                            <td className={`${style.approved} ${style.num}`}>
                                {showSegment(item.homework.approved) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'homework_approved'))}>
                                            {item.homework.approved}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'homework_approved')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.homework.approved
                                }
                            </td>
                            <td className={`${style.opened} ${style.num}`}>
                                {showSegment(item.homework.opened) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'homework_opened'))}>
                                            {item.homework.opened}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'homework_opened')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.homework.opened
                                }
                            </td>
                            <td className={`${style.rejected} ${style.num}`}>
                                {showSegment(item.homework.rejected) ?
                                    <>
                                        <UserLink onClick={() => viewUsers(columnSegment(item, 'homework_rejected'))}>
                                            {item.homework.rejected}
                                        </UserLink>&nbsp;
                                        <SegmentBtn
                                            data={columnSegment(item, 'homework_rejected')}
                                            onClick={downloadUsers}
                                        />
                                    </>
                                    : item.homework.rejected
                                }
                            </td>
                        </tr>
                    )}
                </>
            }
            </tbody>
        </table>
    </div>
}

const DateInput = forwardRef((props, ref) => {
    return (
        <input type="text" style={{ width: '100px' }} {...props} ref={ref}/>
    )
});
