<?php

namespace Mbl\Analytics;

use Exception;

class Segment
{
    private string $type;
    private array $params;

    /**
     * @param string $type
     * @param array $params
     */
    public function __construct( string $type, array $params )
    {
        $this->type = $type;
        $this->params = $params;
    }

    /**
     * @throws Exception
     */
    public function selection(): UserSelection
    {
        $params = $this->params;

        switch ( $this->type ) {
            case 'sources':
                $duration = $params['duration'];
                $units = null;
                if ( isset( $params['is_unlimited'] ) ) {
                    if ( !$params['is_unlimited'] ) {
                        $units = $params['units'];
                    } else {
                        $duration = 'unlimited';
                    }
                }
                $selection = new UserBySourcesByDuration(
                    $params['levels'],
                    $params['source'],
                    new SDateRange( $params['from'], $params['to'] ),
                    $duration,
                    $units,
                    $params['utm_source'] ?? '',
                    $params['utm_medium'] ?? '',
                    $params['utm_campaign'] ?? '',
                    $params['utm_content'] ?? '',
                    $params['utm_term'] ?? '',
                    $params['utm_id'] ?? '',
                );
                break;
            case 'sales':
                $dateRange = new SDateRange( $params['from'], $params['to'] );
                if ( $params['sale'] ) {
                    $selection = new UserHasAccessPurchased( $params['level'], $params['levels'], $dateRange );
                } else {
                    $selection = new UserHasAccessNotPurchased( $params['level'], $params['levels'], $dateRange );
                }
                break;
            case 'expiring':
                $dateRange = new SDateRange( $params['from'], $params['to'] );
                $selection = new UserExpiringAccess(
                    $params['level'],
                    $dateRange,
                    $params['duration'] ?? 'limited',
                    $params['units'] ?? null
                );
                break;
            case 'courses':
                $selection = new class($params) implements UserSelection {
                    private array $params;

                    /**
                     * @param array $params
                     */
                    public function __construct( array $params )
                    {
                        $this->params = $params;
                    }

                    public function request( string $tablePrefix = '' ): string
                    {
                        $course_id = (int)$this->params['course'];
                        $post_id = (int)$this->params['lesson'];
                        $postMeta = get_post_meta( $post_id, '_wpm_page_meta', true );
                        $hasHomework = wpm_array_get( $postMeta, 'is_homework' ) == 'on';
                        $column = $this->params['column'];
                        $course = new Course( $course_id );

                        foreach ( $course->lessons() as $i => $post ) {
                            if ( $post->ID === $post_id ) {
                                $index = $i;
                                break;
                            }
                        }

                        if ( !isset( $index ) ) {
                            return '';
                        }

                        $range = new SDateRange( $this->params['from'], $this->params['to'] );
                        $selection = new UserHasAccessByCourse( $course, $this->params['levels'], $range );

                        $filterCallback = function ( $item ) use ( $hasHomework, $column, $course, $index ) {
                            $lessonData = new LessonData( $item['lessons'][$index], $course );
                            $visibility = $hasHomework && $lessonData->isVisible() && $lessonData->isAvailable();
                            switch ( $column ) {
                                case 'available':
                                    return $lessonData->isAvailable();
                                case 'opened':
                                    return $lessonData->isOpened();
                                case 'no_opened':
                                    return !$lessonData->isOpened();
                                case 'homework_sent':
                                    return $visibility && $lessonData->homeworkIsSent();
                                case 'homework_no_sent':
                                    return $visibility && !$lessonData->homeworkIsSent();
                                case 'homework_approved':
                                    return $visibility && $lessonData->homeworkIsApproved();
                                case 'homework_opened':
                                    return $visibility && $lessonData->homeworkIsOpened();
                                case 'homework_rejected':
                                    return $visibility && $lessonData->homeworkIsRejected();
                                default:
                                    return false;
                            }
                        };

                        $users = (new CourseUsers( $course ))->toArray( $selection, $range );
                        $filtered = array_filter( $users, $filterCallback );
                        $usersIds = implode( ', ', array_map( fn( $user ) => $user['user_id'], $filtered ) );

                        global $wpdb;
                        return "FROM `{$wpdb->prefix}memberlux_term_keys` AS tk$tablePrefix" . PHP_EOL
                            . " WHERE tk$tablePrefix.user_id IN($usersIds)";
                    }

                    public function idsRequest( string $tablePrefix = '' ): string
                    {
                        return "SELECT DISTINCT tk$tablePrefix.user_id {$this->request($tablePrefix)}";
                    }
                };
                break;
            default:
                throw new Exception( 'Wrong segment type' );
        }

        return $selection;
    }
}
